package main

import (
	"context"
	"github.com/disgoorg/disgo"
	"github.com/disgoorg/disgo/bot"
	"github.com/disgoorg/disgo/cache"
	"github.com/disgoorg/disgo/gateway"
	"github.com/getsentry/sentry-go"
	"grow.rievo.dev/discordBots/cmd/vcbot/config"
	"grow.rievo.dev/discordBots/cmd/vcbot/event"
	"log"
	"os"
	"os/signal"
	"syscall"
	"time"
)

// sentry
var release string

func main() {

	err := sentry.Init(sentry.ClientOptions{
		// Either set your DSN here or set the SENTRY_DSN environment variable.
		//Dsn: "",
		// Set TracesSampleRate to 1.0 to capture 100%
		// of transactions for performance monitoring.
		// We recommend adjusting this value in production,
		TracesSampleRate: 1.0,
		Release:          release,
	})
	if err != nil {
		log.Fatalf("sentry.Init: %s", err)
	}
	// Flush buffered events before the program terminates.
	defer sentry.Flush(2 * time.Second)

	log.Printf("INFO: starting vcbot...")
	log.Printf("INFO: disgo version: %v", disgo.Version)

	err = config.LoadAppleList()
	if err != nil {
		log.Fatal(err)
		return
	}

	// permissions: Manage Channels
	// intents:
	client, err := disgo.New(config.Token,
		bot.WithGatewayConfigOpts(
			gateway.WithIntents(gateway.IntentGuildVoiceStates),
			gateway.WithAutoReconnect(true),
		),
		bot.WithCacheConfigOpts(
			cache.WithCaches(
				cache.FlagChannels,
				cache.FlagVoiceStates,
				cache.FlagMembers,
			),
		),
		bot.WithEventListenerFunc(event.JoinEvent),
		bot.WithEventListenerFunc(event.MoveEvent),
		bot.WithEventListenerFunc(event.LeaveEvent),
	)
	if err != nil {
		log.Fatal("error while building vcbot instance: ", err)
		return
	}

	defer client.Close(context.TODO())

	if err = client.OpenGateway(context.TODO()); err != nil {
		log.Fatal("error while connecting to gateway: ", err)
	}

	channels, err := client.Rest().GetGuildChannels(config.RegisterGuildID)
	for _, channel := range channels {
		switch channel.Name() {
		case "🔊-talk":
			config.ChannelVoiceGroupID = channel.ID()
		case "📋-voice":
			config.ChannelVoiceLogID = channel.ID()
		}
	}
	if config.ChannelVoiceGroupID == 0 {
		log.Fatal("couldn't find needed channel")
	}

	log.Printf("ERROR: vcbot is now running. Press CTRL-C to exit.")
	sentry.ConfigureScope(func(scope *sentry.Scope) {
		scope.SetLevel(sentry.LevelDebug)
		sentry.CaptureMessage("DEBUG: vcbot started")
	})

	s := make(chan os.Signal, 1)
	signal.Notify(s, syscall.SIGINT, syscall.SIGTERM, os.Interrupt)
	<-s
}
