package main

import (
	"context"
	"github.com/disgoorg/disgo"
	"github.com/disgoorg/disgo/bot"
	"github.com/disgoorg/disgo/cache"
	"github.com/disgoorg/disgo/gateway"
	"github.com/getsentry/sentry-go"
	"grow.rievo.dev/discordBots/cmd/welcomebot/config"
	"grow.rievo.dev/discordBots/cmd/welcomebot/event"
	"log"
	"os"
	"os/signal"
	"syscall"
	"time"
)

// sentry
var release string

func main() {

	err := sentry.Init(sentry.ClientOptions{
		// Either set your DSN here or set the SENTRY_DSN environment variable.
		//Dsn: "",
		// Set TracesSampleRate to 1.0 to capture 100%
		// of transactions for performance monitoring.
		// We recommend adjusting this value in production,
		TracesSampleRate: 1.0,
		Release:          release,
	})
	if err != nil {
		log.Fatalf("sentry.Init: %s", err)
	}
	// Flush buffered events before the program terminates.
	defer sentry.Flush(2 * time.Second)

	log.Printf("INFO: starting welcomebot...")
	log.Printf("INFO: disgo version: %v", disgo.Version)

	// permissions: Manage Roles
	// intent: Server Members Intent
	client, err := disgo.New(config.Token,
		bot.WithGatewayConfigOpts(
			gateway.WithIntents(gateway.IntentGuildMembers),
			gateway.WithAutoReconnect(true),
		),
		bot.WithCacheConfigOpts(
			cache.WithCaches(
				cache.FlagsNone,
			),
		),
		bot.WithEventListenerFunc(event.JoinEvent),
		bot.WithEventListenerFunc(event.LeaveEvent),
		bot.WithEventListenerFunc(event.ReactionEvent),
	)
	if err != nil {
		log.Fatal("error while building disgo instance: ", err)
		return
	}

	defer client.Close(context.TODO())

	if err = client.OpenGateway(context.TODO()); err != nil {
		log.Fatal("error while connecting to gateway: ", err)
	}

	channels, err := client.Rest().GetGuildChannels(config.RegisterGuildID)
	for _, channel := range channels {
		switch channel.Name() {
		case "🔨-rules":
			config.ChannelRulesMention = channel.Mention()
		case "❕-info":
			config.ChannelInfoMention = channel.Mention()
		case "👋-welcome":
			config.ChannelWelcomeID = channel.ID()
		}
	}
	if config.ChannelRulesMention == "" || config.ChannelInfoMention == "" || config.ChannelWelcomeID == 0 {
		log.Fatal("couldn't find needed channel")
	}

	roles, err := client.Rest().GetRoles(config.RegisterGuildID)
	for _, role := range roles {
		switch role.Name {
		case "Apple":
			config.RoleAppleID = role.ID
		case "Apple Core":
			config.RoleAppleCoreID = role.ID
		}
	}
	if config.RoleAppleID == 0 || config.RoleAppleCoreID == 0 {
		log.Fatal("couldn't find needed role")
	}

	log.Printf("INFO: welcomebot is now running. Press CTRL-C to exit.")
	sentry.ConfigureScope(func(scope *sentry.Scope) {
		scope.SetLevel(sentry.LevelDebug)
		sentry.CaptureMessage("DEBUG: welcomebot started")
	})

	s := make(chan os.Signal, 1)
	signal.Notify(s, syscall.SIGINT, syscall.SIGTERM, os.Interrupt)
	<-s
}
